# Import DQoc HTML from lp:ubuntu-ui-toolkit
import os, sys, re

from django.core.management.base import BaseCommand, CommandError
from django.core.urlresolvers import reverse
from django.conf import settings

from optparse import make_option
import xml.etree.ElementTree as ET

from developer_network.apidocs.models import *
from developer_network.common.models import *

__all__ = (
    'Command',
)

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option(
            "-i",
            "--index",
            dest="index",
            help="QDoc .index file to import documentation from",
        ),

        make_option(
            "-s",
            "--section",
            dest="section",
            help="Section the documentation if from",
        ),
        make_option(
            "-r",
            "--release",
            dest="version",
            help="Version of the topic the documentation if from",
        ),
        make_option(
            "-t",
            "--topic",
            dest="topic",
            help="Topic the documentation if from",
        ),
    )

    def parse_line(self, line, source_filename):
        try:
            clean_line = line
            source_line = line
            if source_filename in clean_line:
                clean_line = line.replace('href="'+source_filename+'#', 'href="#')
                source_line = clean_line
                
            link_matcher = re.compile('href="([\w\-\.]+)\.html(#[\w\-\.]+)?"')
            match = link_matcher.search(source_line)
            while match:
                if self.verbosity >= 3:
                    print "Matched line [%s:%s]: %s " % (match.start(), match.end(), source_line)
                filename = '%s.html' % match.group(1)
                if filename in self.page_map:
                    new_link = settings.URL_ROOT + reverse('web.views.element_view', args=(self.topic.slug, self.version.slug, self.page_map[filename]))
                    clean_line = clean_line.replace('href="'+filename, 'href="'+new_link)
                    # Found a reference to a page, add it to the list to import later
                    if filename not in self.page_refs:
                        if self.verbosity >= 2:
                            print 'New page reference in %s: %s with %s' % (source_filename, match.group(1), match.group(2))
                        self.page_refs[filename] = 1

                if filename in self.qml_class_map:
                    new_link = settings.URL_ROOT + reverse('web.views.element_view', args=(self.topic.slug, self.version.slug, self.qml_class_map[filename]))
                    clean_line = clean_line.replace('href="'+filename, 'href="'+new_link)

                # Continue trying to match on this line
                source_line = source_line[match.end():]
                match = link_matcher.search(source_line)
            
            return clean_line
        except Exception, e:
            print "Error cleaning line: " + line
            import pdb; pdb.set_trace()
            return line
        
    def parse_filename(self, filename):
        pass
        
    def parse_href(self, href):
        pass
        
    def parse_pagename(self, pagename):
        if pagename.endswith('.html'):
            pagename = pagename[:-5]
        return pagename.replace('/', '-').replace(' ', '_')
        
    def parse_classname(self, classname):
        return classname
        
    def parse_namespace(self, namespace):
        if namespace == 'QtQuick' and self.source.endswith('/qtqml.index'):
            return 'QtQml'
        else:
            return namespace
        
    def handle(self, *args, **options):
        self.verbosity = int(options['verbosity'])
        self.topic = Topic.objects.get(slug=options['topic'])
        self.version = Version.objects.get(slug=options['version'], topic=self.topic)
        self.section = Section.objects.get(name=options['section'], topic_version=self.version)

        self.source = options['index']
        if not os.path.exists(self.source):
            print "Source index file not found"
            exit(1)
            
        DOC_ROOT = os.path.dirname(self.source)
        
        tree = ET.parse(self.source)
        root = tree.getroot()
        
        # Map document filenames to QML class names
        self.qml_class_map = {}
        for qmlclass in root.find('namespace').iter('qmlclass'):
            classname = self.parse_classname(qmlclass.get('name'))
            ns_name = self.parse_namespace(qmlclass.get('qml-module-name'))
            if ns_name is not None:
                self.qml_class_map[qmlclass.get('href')] = ns_name+'.'+classname
            else:
                self.qml_class_map[qmlclass.get('href')] = classname

        # Map document filename to page names
        self.page_map = {}
        for pagenode in root.find('namespace').iter('page'):
            if pagenode.get('subtype') != 'page':
                continue
            pagename = self.parse_pagename(pagenode.get('name'))
            ns_name = self.parse_namespace(pagenode.get('module'))
            if ns_name is not None:
                self.page_map[pagenode.get('href')] = pagenode.get('module')+'.'+pagename
            else:
                self.page_map[pagenode.get('href')] = pagename

        # Keep track of pages that are references so they can be imported later
        self.page_refs = {}
        
        # Import QML class documentation
        for qmlclass in root.find('namespace').iter('qmlclass'):
            classname = self.parse_classname(qmlclass.get('name'))
            ns_name = self.parse_namespace(qmlclass.get('qml-module-name'))
            if ns_name is not None:
                namespace, created = Namespace.objects.get_or_create(name=ns_name, platform_section=self.section)
            else:
                namespace = None

            element, created = Element.objects.get_or_create(name=classname, section=self.section, namespace=namespace)
            if namespace is not None:
                element.fullname = namespace.name + '.' + classname
            else:
                element.fullname = classname
                
            if self.verbosity >= 1:
                print 'Element: ' + element.fullname
                
            doc_file = os.path.join(DOC_ROOT, qmlclass.get('href'))
            doc_handle = open(doc_file)
            doc_data = doc_handle.readlines()
            doc_handle.close()
            
            doc_start = 2
            doc_end = -2
            for i, line in enumerate(doc_data):
                if '<h1 class="title"' in line:
                    doc_start = i+1
                if '<div class="footer"' in line:
                    doc_end = i-1
            if self.verbosity >= 3:
                print "Doc range: %s:%s" % (doc_start, doc_end)

            try:
                brief_start = doc_data.index('<!-- $$$'+element.name+'-brief -->\n')
                element.description = self.parse_line(doc_data[brief_start+1][3:-35], qmlclass.get('href'))
                if len(element.description) >= 256:
                    element.description = element.description[:252]+'...'
            except ValueError:
                pass

            try:
                # Change the content of the docs 
                cleaned_data = ''
                for line in doc_data[doc_start:doc_end]:
                    if line == '' or line == '\n':
                        continue
                    if 'List of all members, including inherited members' in line:
                        continue
                    line = self.parse_line(line, qmlclass.get('href'))
                    if isinstance(line, unicode):
                        line = line.encode('ascii', 'replace')
                    cleaned_data += line
                    
                element.data = cleaned_data
            except Exception, e:
                print "Parsing content failed:"
                print e
                #import pdb; pdb.set_trace()
                
            element.save()
            
        while 1 in self.page_refs.values():
            if self.verbosity >= 2:
                print "Page refs needing import: %s'" % self.page_refs.keys()
            for pagenode in root.find('namespace').iter('page'):
                # Only import pages that are referenced elsewhere
                if pagenode.get('href') not in self.page_refs:
                    continue

                # Remove this page from the reference list so it won't be imported again
                self.page_refs[pagenode.get('href')] = 0
                
                pagename = self.parse_pagename(pagenode.get('name'))
                ns_name = self.parse_namespace(pagenode.get('module'))
                if ns_name is not None:
                    namespace, created = Namespace.objects.get_or_create(name=ns_name, platform_section=self.section)
                else:
                    namespace = None
                    
                page, created = Page.objects.get_or_create(slug=pagename, title=pagenode.get('title'), section=self.section, namespace=namespace)
                if namespace is not None:
                    page.fullname = namespace.name + '.' + pagename
                else:
                    page.fullname = pagename

                if self.verbosity >= 1:
                    print 'Page: ' + page.slug
                
                doc_file = os.path.join(DOC_ROOT, pagenode.get('href'))
                doc_handle = open(doc_file)
                doc_data = doc_handle.readlines()
                doc_handle.close()
                
                doc_start = 2
                doc_end = -2
                for i, line in enumerate(doc_data):
                    if '<body' in line:
                        doc_start = i+1
                    if '<div class="footer"' in line:
                        doc_end = i-1
                if self.verbosity >= 3:
                    print "Doc range: %s:%s" % (doc_start, doc_end)

                try:
                    # Change the content of the docs 
                    cleaned_data = ''
                    for line in doc_data[doc_start:doc_end]:
                        if line == '' or line == '\n':
                            continue
                        if '<h1 class="title">' in line:
                            continue
                        line = self.parse_line(line, pagenode.get('href'))
                        if isinstance(line, unicode):
                            line = line.encode('ascii', 'replace')
                        cleaned_data += line
                        
                    page.data = cleaned_data
                except Exception, e:
                    print "Parsing content failed:"
                    print e
                    #continue
                    #import pdb; pdb.set_trace()
                page.save()

