from django.shortcuts import render_to_response, get_object_or_404
from django.http import HttpResponse, HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.template import loader, RequestContext

from common.models import Topic, Version, Section
from apidocs.models import Namespace, Element, Page
from related.models import Snippet, Link, LINK_QUESTION, LINK_TUTORIAL

# Create your views here.
def overview(request):
    topics = Topic.objects.all()
    
    context = {
        'topics': topics,
    }
    return render_to_response('web/overview.html', context, RequestContext(request))
    
def topic_view(request, topic_name):
    topic = get_object_or_404(Topic, slug=topic_name)
    
    context = {
        'topic': topic,
    }
    return render_to_response('web/distro.html', context, RequestContext(request))

def version_view(request, topic_name, release_version):
    version = get_object_or_404(Version, topic__slug=topic_name, slug=release_version)

    sections = version.section_set.all()
    first_column = []
    second_column = []
    total_items = 0
    sorted_sections = []
    for section in sections:
        section_count = section.namespace_set.count()
        section_count += section.free_element_set().count()
        if section_count == 0:
            continue
        total_items += section_count
        i = 0
        for s in sorted_sections:
            if (s.namespace_set.count() + s.free_element_set().count()) > section_count:
                i += 1
            else:
                break
        sorted_sections.insert(i, section)
        
    first_column_items = 0
    for section in sorted_sections:
        if first_column_items <= (total_items / 2):
            first_column.append(section)
            first_column_items += (section.namespace_set.count() + section.free_element_set().count())
        else:
            second_column.append(section)

    context = {
        'sidenav': topic_name,
        'topic': version.topic,
        'version': version,
        'first_column': first_column,
        'second_column': second_column,
    }
    return render_to_response('web/release.html', context, RequestContext(request))

def namespace_view(request, topic_name, release_version, namespace_name):
    try:
        namespace = Namespace.objects.get(platform_section__topic_version__slug=release_version, platform_section__topic_version__topic__slug=topic_name, name=namespace_name)
    except Namespace.DoesNotExist:
        return page_view(request, topic_name, release_version, namespace_name)
    links = Link.objects.filter(rendered_element=namespace)
    questions = [link for link in links if link.link_type is LINK_QUESTION]
    tutorials = [link for link in links if link.link_type is LINK_TUTORIAL]
    
    context = {
        'sidenav': topic_name,
        'topic': namespace.platform_section.topic_version.topic,
        'version': namespace.platform_section.topic_version,
        'namespace': namespace,
        'questions': questions,
        'tutorials': tutorials
    }
    return render_to_response('web/namespace.html', context, RequestContext(request))

def page_view(request, topic_name, release_version, page_fullname):
    page = get_object_or_404(Page, section__topic_version__slug=release_version, section__topic_version__topic__slug=topic_name, fullname=page_fullname)

    links = Link.objects.filter(rendered_element=page)
    questions = [link for link in links if link.link_type is LINK_QUESTION]
    tutorials = [link for link in links if link.link_type is LINK_TUTORIAL]
    
    context = {
        'sidenav': topic_name,
        'topic': page.section.topic_version.topic,
        'version': page.section.topic_version,
        'page': page,
        'questions': questions,
        'tutorials': tutorials
    }
    return render_to_response('web/page.html', context, RequestContext(request))

def element_view(request, topic_name, release_version, element_fullname):
    try:
        element = Element.objects.get(section__topic_version__slug=release_version, section__topic_version__topic__slug=topic_name, fullname=element_fullname)
    except Element.DoesNotExist:
        return namespace_view(request, topic_name, release_version, element_fullname)
    snippets = Snippet.objects.filter(rendered_element=element)
    links = Link.objects.filter(rendered_element=element)

    questions = [link for link in links if link.link_type is LINK_QUESTION]
    tutorials = [link for link in links if link.link_type is LINK_TUTORIAL]
    
    context = {
        'sidenav': topic_name,
        'topic': element.section.topic_version.topic,
        'version': element.section.topic_version,
        'element': element,
        'snippets': snippets,
        'questions': questions,
        'tutorials': tutorials
    }
    return render_to_response('web/element.html', context, RequestContext(request))
    
def search(request, topic_name, release_version):
    version = get_object_or_404(Version, topic__slug=topic_name, slug=release_version)
    query = request.GET.get('query', '')
    results = Element.objects.filter(section__topic_version=version, name__icontains=query).order_by('section', 'fullname')
    
    context = {
        'sidenav': topic_name,
        'topic': version.topic,
        'version': version,
        'query': query,
        'results': results
    }
    return render_to_response('web/search.html', context, RequestContext(request))

